@@ -77,4 +77,21 @@ class DefaultRequirements(SuiteRequirements):
         """Target driver reflects the name of primary key constraints."""
 
         return exclusions.fails_on_everything_except(
-            'postgresql', 'oracle', 'mssql', 'sybase')
+            'postgresql', 'oracle', 'mssql', 'sybase',
+            lambda config: (
+                util.sqla_110 and exclusions.against(config, "sqlite")
+            )
+        )
+
+    @property
+    def postgresql_uuid_ossp(self):
+        def check_uuid_ossp(config):
+            if not exclusions.against(config, "postgresql"):
+                return False
+            try:
+                config.db.execute("SELECT uuid_generate_v4()")
+                return True
+            except:
+                return False
+
+        return exclusions.only_if(check_uuid_ossp)
